"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._processTree = exports._filePathFromProcessId = exports.GetCurrentProcessId = exports.EnumProcesses = exports.ProcessSnapshot_cb = exports.ProcessSnapshot = exports.GetDrives = exports.DriveToDosDevice = exports.QueryFullProcessImageName = void 0;
//@ts-ignore
const nprocess_node_1 = __importDefault(require("../../nprocess.node"));
//--------------原生大写开头
/**
 * 进程ID取运行路径
 * @param ProcessId 进程id
 */
function QueryFullProcessImageName(ProcessId) {
    //test:ok
    return nprocess_node_1.default.QueryFullProcessImageName(ProcessId);
}
exports.QueryFullProcessImageName = QueryFullProcessImageName;
/**
 * 系统盘符转换为DOS设备描述
 * @param Drive 盘符,例如:C,D
 */
function DriveToDosDevice(Drive) {
    //test:ok
    return nprocess_node_1.default.DriveToDosDevice(Drive);
}
exports.DriveToDosDevice = DriveToDosDevice;
/**
 * 获取所有盘符
 */
function GetDrives() {
    //test:ok
    let arr = nprocess_node_1.default.GetDrives();
    return arr.map((v) => {
        return String.fromCharCode(v);
    });
}
exports.GetDrives = GetDrives;
/**
 * 进程快照
 * @returns
 */
function ProcessSnapshot() {
    //test:ok
    return nprocess_node_1.default.ProcessSnapshot();
}
exports.ProcessSnapshot = ProcessSnapshot;
/**
 * 进程快照 回调形式
 * @returns
 */
function ProcessSnapshot_cb(callback) {
    //test:ok
    return nprocess_node_1.default.ProcessSnapshot_cb(callback);
}
exports.ProcessSnapshot_cb = ProcessSnapshot_cb;
/**
 * 提供了进程快照和基于快照的进程树，内部做了一个完整的循环。
 * 如果要对ProcessId进程处理或操作。建议使用EnumProcesses，因为只做了一次循环。
 * 回调传回Id可以对Id做更多操作。但快照有个好处,没有权限方面限制。
 * @function 枚举进程,Psapi.dll->EnumProcesses
 * @param callback EnumProcessCallback:(processId:number):any
 * @returns 如果函数调用成功返回真
 * @returns Returns true if the function call succeeds
 */
function EnumProcesses(callback) {
    return nprocess_node_1.default.EnumProcesses(callback);
}
exports.EnumProcesses = EnumProcesses;
/**
 * 取当前进程ID，但并不确定用在electron上是什么效果。
 */
function GetCurrentProcessId() {
    return nprocess_node_1.default.GetCurrentProcessId();
}
exports.GetCurrentProcessId = GetCurrentProcessId;
//--------------------包装后方法，小写开头
/**
 * 进程ID取文件路径
 * @param ProcessId
 * @returns
 */
function _filePathFromProcessId(ProcessId) {
    let path = QueryFullProcessImageName(ProcessId);
    let drives = GetDrives();
    for (let i = 0; i < drives.length; i++) {
        let drive = drives[i];
        let _dos = DriveToDosDevice(drive);
        if (path.includes(_dos)) {
            return path.replace(_dos, drive + ':');
        }
    }
    return '';
}
exports._filePathFromProcessId = _filePathFromProcessId;
/**
 * 生成进程树
 * @returns
 */
function _processTree() {
    let temp = {}, tree = {};
    ProcessSnapshot_cb((p) => {
        temp[p.Id] = p;
    });
    for (let i in temp) {
        if (temp[i].ParentId && temp[temp[i].ParentId]) {
            if (!temp[temp[i].ParentId].children) {
                temp[temp[i].ParentId].children = new Object();
            }
            temp[temp[i].ParentId].children[temp[i].Id] = temp[i];
        }
        else {
            tree[temp[i].Id] = temp[i];
        }
    }
    temp = null;
    return tree;
}
exports._processTree = _processTree;
