export interface RECT {
    left: number;
    top: number;
    right: number;
    bottom: number;
}
export declare const GWL: {
    EXSTYLE: number;
    HINSTANCE: number;
    HWNDPARENT: number;
    ID: number;
    STYLE: number;
    USERDATA: number;
};
export declare const SWP: {
    ASYNCWINDOWPOS: number;
    DEFERERASE: number;
    DRAWFRAME: number;
    FRAMECHANGED: number;
    HIDEWINDOW: number;
    NOACTIVATE: number;
    NOCOPYBITS: number;
    NOMOVE: number;
    NOOWNERZORDER: number;
    NOREDRAW: number;
    NOSENDCHANGING: number;
    NOSIZE: number;
    NOZORDER: number;
    SHOWWINDOW: number;
};
/**
 * @function 获取窗口DPI
 */
export declare function GetDpiForWindow(hWnd: number): any;
/**
 * @function 获取系统DPI
 */
export declare function GetDpiForSystem(): any;
/**
 * @function 寻找窗口
 * @param className 窗口类名,为空则返回第一个与类名匹配的窗口
 * @param className "" returns the first window that matches the className
 * @param windowName 窗口标题,为空则返回第一个与标题匹配的窗口
 * @param windowName "" returns the first window that matches the windowName
 * @returns hWnd 窗口句柄
 */
export declare function FindWindowA(className?: string, windowName?: string): any;
/**
 * @function 寻找窗口
 * @param hWndParent 为0等同于C++null
 * @param hWndParent 0 equals C++ NULL
 * @param hWndChildAfter 为0等同于C++null
 * @param hWndChildAfter 0 equals C++ NULL
 * @param className 窗口类名,为空则返回第一个与类名匹配的窗口
 * @param className "" returns the first window that matches the className
 * @param windowName 窗口标题,为空则返回第一个与标题匹配的窗口
 * @param windowName "" returns the first window that matches the windowName
 * @returns hWnd 窗口句柄
 */
export declare function FindWindowExA(hWndParent: number, hWndChildAfter: number, className: string, WindowName: string): number;
/**
 * @function 获取客户区矩形
 * @param hWnd
 * @returns 返回矩形，失败返回null
 * @returns Returns a rectangle. Failure returns null
 */
export declare function GetClientRect(hWnd: number): RECT | null;
/**
 * @function 获取窗口矩形
 * @param hWnd
 * @returns 返回矩形，失败返回null
 * @returns Returns a rectangle. Failure returns null
 */
export declare function GetWindowRect(hWnd: number): RECT | null;
/**
 * @function 更新窗口,成功返回真，反之。
 * @param hWnd
 */
export declare function UpdateWindow(hWnd: number): any;
/**
 *
 * @param hWnd
 * @param gaFlags The ancestor to be retrieved.
 * @param gaFlags 要检索的祖先。
 */
export declare function GetAncestor(hWnd: number, gaFlags: number): number;
/**
 * @function Limit mouse area
 * @function 限制鼠标区域
 * @param Rect null为解除
 */
export declare function ClipCursor(Rect: RECT | null): boolean;
/**
 * @function 取限制区矩形
 */
export declare function GetClipCursor(): RECT | null;
/**
 * @function 获取鼠标详情(位置,是否隐藏,指针句柄)
 */
export declare function GetCursorInfo(): {
    ptScreenPos: {
        x: number;
        y: number;
    };
    flags: number;
    hCursor: number;
};
/**
 *  @function 当前进程下窗口隐藏或显示鼠标(electron也可以cursor:none给元素设置)
 *  @function Hide or show mouse in current process window (electron can also be set to cursor:none for elements)
 */
export declare function ShowCursor(bShow: boolean): number;
/**
 *
 * @param hWnd
 * @param nIndex GWL_常量
 * @param dwNewLong
 * @return If the function succeeds, the return value is the requested value. And vice versa is zero.
 * @return 如果函数成功，返回值就是请求值。反之为零。
 */
export declare function SetWindowLongPtrA(hWnd: number, nIndex: number, dwNewLong: number): number;
export declare function GetWindowLongPtrA(hWnd: number, nIndex: number): number;
/**
 * @function 设置窗口透明度，需要设置扩展样式WS_EX_LAYERED
 * @function The extension style WS EX LAYERED needs to be set
 * @param hWnd
 * @param colorKey 颜色值
 * @param alpha  0 completely transparent,255 completely opaque
 * @param alpha  0完全透明,255完全不透明
 * @param flags
 */
export declare function SetLayeredWindowAttributes(hWnd: number, colorKey: number, alpha: number, flags: number): boolean;
/**
 * @function 更改窗口的大小,位置
 * @function Change window size, position
 * @param hWnd 窗口句柄
 * @param hWndInsertAfter 该窗口句柄之后或HWND_常量值
 * @param hWndInsertAfter After the window handle or HWND constant value
 * @param X
 * @param Y
 * @param cx 宽度 width
 * @param cy 高度 height
 * @param uFlags 窗口大小和定位标志。SWP_常量值可组合
 */
export declare function SetWindowPos(hWnd: number, hWndInsertAfter: number, X: number, Y: number, cx: number, cy: number, uFlags: number): boolean;
/**
 * @function 获取分层窗口的不透明度和颜色值
 * @function Gets the opacity and color values of the layered window
 * @param hWnd
 */
export declare function GetLayeredWindowAttributes(hWnd: number): {
    colorKey: number;
    alpha: number;
    flags: number;
} | false;
/**
 * @function 显示窗口
 * @param hWnd
 * @param CmdShow 显示方式
 */
export declare function ShowWindow(hWnd: number, CmdShow: number): number;
/**
 * @function 投递消息
 * @param hWnd
 * @param msg 消息值
 * @param wParam
 * @param lParam
 */
export declare function PostMessageA(hWnd: number, msg: number, wParam: number, lParam: number): number;
/**
 * @function 发送消息
 * @param hWnd
 * @param msg 消息值
 * @param wParam
 * @param lParam
 */
export declare function SendMessageA(hWnd: number, msg: number, wParam: number, lParam: number): number;
/**
 * 取窗口标题
 * @param hWnd
 */
export declare function GetWindowText(hWnd: number): string;
/**
 * 聚焦窗口
 * @param hWnd
 */
export declare function SetForegroundWindow(hWnd: number): boolean;
/**
 * 取点位置窗口
 * @param x
 * @param y
 */
export declare function WindowFromPoint(x: number, y: number): any;
/**
 * @function 窗口是否有效
 * @param hWnd
 */
export declare function IsWindow(hWnd: number): Boolean;
export declare function GetWindowInfo(hWnd: number): {
    cxWindowBorders: number;
    cyWindowBorders: number;
    atomWindowType: number;
    dwStyle: number;
    dwExStyle: number;
    dwWindowStatus: number;
    rcWindow: RECT;
    rcClient: RECT;
    wCreatorVersion: number;
} | false;
/**
 * 具体查看文档EnumWindows，函数调用错误会设置Error。
 * @returns 返回所有顶级窗口。。
 */
export declare function EnumWindows(): number[] | false;
/**
 * 获取桌面句柄
 */
export declare function GetDesktopWindow(): number;
/**
 * 获取最后一条错误。
 */
export declare function GetLastError(): number;
/**
 * 获取聚焦窗口句柄
 */
export declare function GetForegroundWindow(): any;
/**
 * @function EnumDesktopWindows 基于桌面为根基进行枚举窗口
 * @param desktopHwnd 桌面句柄。如果没有提供desktopHwnd,默认为NULL。表示当前桌面。
 */
export declare function EnumDesktopWindows(desktopHwnd?: number): number[] | false;
/**
 * 取窗口线程ID和进程ID
 * @param hWnd
 */
export declare function GetWindowThreadProcessId(hWnd: number): {
    ProcessId: number;
    ThreadId: number;
};
/**
 * 窗口是否启用，窗口允许接受键盘或鼠标输入返回true，反之。
 * 子窗口只有在被允许并且可见时才可接受输入
 * @param hWnd
 */
export declare function IsWindowEnabled(hWnd: number): boolean;
/**
 * 窗口是否可见
 * @param hWnd
 */
export declare function IsWindowVisible(hWnd: number): boolean;
/**
 * 窗口是否为Unicode窗口
 * @param hWnd
 */
export declare function IsWindowUnicode(hWnd: number): boolean;
/**
 * 窗口是否最大
 * @param hWnd
 */
export declare function IsZoomed(hWnd: number): boolean;
/**
 * 获取窗口类名
 * @param hWnd
 */
export declare function GetClassName(hWnd: number): string;
export declare function GetClassLongPtrA(hWnd: number, index: number): any;
export declare function SetClassLongPtrA(hWnd: number, index: number, newLong: number): any;
/**
 * 获取窗口图标句柄,使用SendMessageTimeout和GetClassLong,有如果失败会有40ms的延迟
 * 可自行更改获取方式例如单独的GetClassLong
 * @param hWnd
 */
export declare function GetWindowHicon(hWnd: number): number;
/**
 * 获取窗口
 * GW_HWNDFIRST=0
 * GW_HWNDLAST=1
 * GW_HWNDNEXT=2
 * GW_HWNDPREV=3
 * GW_OWNER=4
 * GW_CHILD=5
 * GW_ENABLEDPOPUP=6
 * @param hWnd
 * @param cmd
 */
export declare function GetWindow(hWnd: number, cmd: number): number;
/**
 * 启用或禁用窗口
 * @param hWnd
 * @param flag
 */
export declare function EnableWindow(hWnd: number, flag: boolean): any;
/**
 * @function 枚举窗口的子窗口句柄
 * @function Enumerates the handle to a child window of a window
 * @param ParentHWnd 如果为0，则效果等同于EnumWindows
 * @param ParentHWnd If 0, the effect is equivalent to EnumWindows
 */
export declare function EnumChildWindows(ParentHWnd: number): any;
/**
 * @param flags If it is not zero, call SetWindowPos and redraw the preset flag。
 * SWP.NOSIZE | SWP.NOMOVE | SWP.NOZORDER | SWP.NOACTIVATE
 * @returns Returns true on success, or the window style value before the call
 */
export declare function _modifyStyle(hWnd: number, { addStyle, removeStyle }?: {
    addStyle?: number | undefined;
    removeStyle?: number | undefined;
}, flags?: boolean | number): true | number;
/**
 * @param flags If it is not zero, call SetWindowPos and redraw the preset flag。
 * SWP.NOSIZE | SWP.NOMOVE | SWP.NOZORDER | SWP.NOACTIVATE
 * @returns Returns true on success, or the window style value before the call
 */
export declare function _modifyStyleEx(hWnd: number, { addStyle, removeStyle }?: {
    addStyle?: number | undefined;
    removeStyle?: number | undefined;
}, flags?: true | number): true | number;
/**
 * @function Fuzzy search window
 * @param windowName Part of the window title to search, regardless of case
 * @param className The class name must be written in full, emptyString Ignore
 * @function 模糊搜索窗口
 * @param windowName 要搜索的窗口部分窗口标题,无视大小写
 * @param className 类名必须写明,空文本则忽略
 */
export declare function _findWindowFuzzy(windowName: string, className?: string): number;
/**
 * @function 设置窗口透明度
 * @param hWnd
 * @param alpha 不透明值,0完全透明,255完全不透明
 * @param through 窗口是否可穿透
 * @param colorKey
 */
export declare function _setWindowTransparency(hWnd: number, alpha: number, through?: boolean, colorKey?: number | false): boolean;
/**
 * @function 取消窗口透明,返回真成功否则返回之前样式值.
 * @function Cancel window transparency, return true success otherwise return previous style value.
 * @param hWnd
 */
export declare function _cancelWindowTransparency(hWnd: number): number | true;
/**
 * 获取窗口不透明度,只有调用SetLayeredWindowAttributes后才会正确返回
 * 调用upLayeredWindowAttributes后失效
 * @param hWnd
 */
export declare function _getWindowTransparency(hWnd: number): number | false;
/**
 * @function 置顶窗口
 * @function topmost window
 * @param hWnd
 * @param topmost
 */
export declare function _setWindowTopmost(hWnd: number, topmost?: boolean): boolean;
/**
 * @function 取鼠标位置处窗口句柄
 * @function Gets the window handle at the mouse position
 */
export declare function _windowFromCursorPoint(): number;
/**
 * 取窗口是否顶置
 * @param hWnd
 */
export declare function _isTopMost(hWnd: number): number | false;
/**
 * 窗口是否添加分层特效。
 */
export declare function _isTransparency(hWnd: number): number | false;
